@@ -42,7 +42,7 @@
 /* Define this to get some logging all the time */
 /* #define G_SPAWN_WIN32_DEBUG */
 
-#include <config.h>
+#include "config.h"
 
 #include "glib.h"
 #include "gprintfint.h"
@@ -199,10 +199,7 @@ protect_argv (gchar  **argv,
 GQuark
 g_spawn_error_quark (void)
 {
-  static GQuark quark = 0;
-  if (quark == 0)
-    quark = g_quark_from_static_string ("g-exec-error-quark");
-  return quark;
+  return g_quark_from_static_string ("g-exec-error-quark");
 }
 
 gboolean
