@@ -62,11 +62,6 @@
 
 
 
-#define ISDIGIT(Type)	IS ((Type),				\
-			    OR (G_UNICODE_DECIMAL_NUMBER,	\
-			    OR (G_UNICODE_LETTER_NUMBER,	\
-			    OR (G_UNICODE_OTHER_NUMBER,		0))))
-
 #define ISALPHA(Type)	IS ((Type),				\
 			    OR (G_UNICODE_LOWERCASE_LETTER,	\
 			    OR (G_UNICODE_UPPERCASE_LETTER,	\
@@ -320,8 +315,8 @@ g_unichar_istitle (gunichar c)
   unsigned int i;
   for (i = 0; i < G_N_ELEMENTS (title_table); ++i)
     if (title_table[i][0] == c)
-      return 1;
-  return 0;
+      return TRUE;
+  return FALSE;
 }
 
 /**
@@ -337,7 +332,7 @@ g_unichar_isxdigit (gunichar c)
 {
   return ((c >= 'a' && c <= 'f')
 	  || (c >= 'A' && c <= 'F')
-	  || ISDIGIT (TYPE (c)));
+	  || (TYPE (c) == G_UNICODE_DECIMAL_NUMBER));
 }
 
 /**
@@ -514,7 +509,12 @@ g_unichar_toupper (gunichar c)
 	  return g_utf8_get_char (p);
 	}
       else
-	return val ? val : c;
+        {
+	  /* Some lowercase letters, e.g., U+000AA, FEMININE ORDINAL INDICATOR,
+	   * do not have an uppercase equivalent, in which case val will be
+	   * zero. */
+	  return val ? val : c;
+	}
     }
   else if (t == G_UNICODE_TITLECASE_LETTER)
     {
@@ -551,7 +551,11 @@ g_unichar_tolower (gunichar c)
 	  return g_utf8_get_char (p);
 	}
       else
-	return val ? val : c;
+	{
+	  /* Not all uppercase letters are guaranteed to have a lowercase
+	   * equivalent.  If this is the case, val will be zero. */
+	  return val ? val : c;
+	}
     }
   else if (t == G_UNICODE_TITLECASE_LETTER)
     {
@@ -823,11 +827,17 @@ real_toupper (const gchar *str,
 		  for (i = 0; i < G_N_ELEMENTS (title_table); ++i)
 		    {
 		      if (title_table[i][0] == c)
-			val = title_table[i][1];
+			{
+			  val = title_table[i][1];
+			  break;
+			}
 		    }
 		}
 
-	      len += g_unichar_to_utf8 (val, out_buffer ? out_buffer + len : NULL);
+	      /* Some lowercase letters, e.g., U+000AA, FEMININE ORDINAL INDICATOR,
+	       * do not have an uppercase equivalent, in which case val will be
+	       * zero. */
+	      len += g_unichar_to_utf8 (val ? val : c, out_buffer ? out_buffer + len : NULL);
 	    }
 	}
       else
@@ -1007,11 +1017,16 @@ real_tolower (const gchar *str,
 		  for (i = 0; i < G_N_ELEMENTS (title_table); ++i)
 		    {
 		      if (title_table[i][0] == c)
-			val = title_table[i][2];
+			{
+			  val = title_table[i][2];
+			  break;
+			}
 		    }
 		}
 
-	      len += g_unichar_to_utf8 (val, out_buffer ? out_buffer + len : NULL);
+	      /* Not all uppercase letters are guaranteed to have a lowercase
+	       * equivalent.  If this is the case, val will be zero. */
+	      len += g_unichar_to_utf8 (val ? val : c, out_buffer ? out_buffer + len : NULL);
 	    }
 	}
       else
